@@ -4,7 +4,7 @@ import numpy as np
 from plotnine import *
 from PIL import Image
 
-from ATACFragQC import __version__
+from __init__ import __version__
 
 class ArgumentList:
     file_bam = ""
@@ -13,12 +13,12 @@ class ArgumentList:
     group_marker = ""
     quality = 5
     isize = 147
-    range = [1000, 100]
     cn_len = 10
     chr_filter = ""
     chr_list = ""
     calc_mode = "F"
     pic_list = "a,b,c"
+    widthtss = 900
     def __init__(self):
         self.file_bam = ""
         self.file_ref = ""
@@ -26,12 +26,12 @@ class ArgumentList:
         self.group_marker = ""
         self.quality = 5
         self.isize = 147
-        self.range = [1000, 100]
         self.cn_len = 10
         self.calc_mode = "F"
         self.chr_filter = ""
         self.chr_list = ""
         self.pic_list = "a,b,c"
+        self.widthtss = 900
 
 def chr_cmp(a, b):
     sa = str(a)
@@ -53,8 +53,12 @@ def bedScan(args):
     (pathname, extension) = os.path.splitext(args.file_bam)
     (filepath, filename) = os.path.split(pathname)
     if not os.path.isfile(args.file_bam+".bai"):
-        print("There is no index file for the bam...")
-        return
+        print("Creating the index file of bam...")
+        if os.system("which samtools") == 0:
+            os.system("samtools index "+args.file_bam)
+        else:
+            print("Failed, please check your environment variables...")
+            return
     print("Processing the reference...")
     ref_raw = pd.read_table(args.file_ref, comment="#", header=None, dtype={0:str})
     ref_raw.columns = ["seq_id", "source", "type", "start", "end", "score", "strand", "phase", "attributes"]
@@ -85,13 +89,13 @@ def bedScan(args):
         return
     chr_list = sorted(list(set(chr_list).difference(set(args.chr_filter.split(",")))), key=functools.cmp_to_key(chr_cmp))
     chr_list_frag = [x for x in chr_list if re.match(r".*[Mm]+,*", x) == None]
-    ref = ref_raw[(ref_raw["type"] == type_test) & (ref_raw["strand"] != "-") & ref_raw["seq_id"].str.match("^"+chr_list_frag[0]+"$") & (ref_raw["start"] > args.range[0])]
-    for term in chr_list_frag[1:]:
-        ref = ref.append(ref_raw[(ref_raw["type"] == type_test) & (ref_raw["strand"] != "-") & ref_raw["seq_id"].str.match("^"+term+"$") & (ref_raw["start"] > args.range[0])])
+    ref = pd.DataFrame()
+    for term in chr_list_frag:
+        ref = pd.concat([ref, ref_raw[(ref_raw["type"] == type_test) & (ref_raw["strand"] != "-") & ref_raw["seq_id"].str.match("^"+term+"$") & (ref_raw["start"] > args.widthtss+100)]])
     ref = ref[["seq_id", "strand", "start", "end"]].sort_values(by=["seq_id", "start", "strand"])
     ref = ref.drop_duplicates(subset=["seq_id", "start", "strand"], keep="first")
-    ref.loc[ref["strand"] == "+", "start"] -= args.range[0]
-    ref.loc[ref["strand"] == "+", "end"] = ref.loc[ref["strand"] == "+", "start"] + args.range[0]*2
+    ref.loc[ref["strand"] == "+", "start"] -= args.widthtss+100
+    ref.loc[ref["strand"] == "+", "end"] = ref.loc[ref["strand"] == "+", "start"] + (args.widthtss+100)*2
     ref.index = list(range(ref.index.size))
     
     print("Scaning the distribution of fragments...")
@@ -111,7 +115,7 @@ def bedScan(args):
     print("Scaning the fragments around TSSs...")
     dist_count = []
     dist_count_group = {}
-    tss_range = args.range[0]*2+1
+    tss_range = (args.widthtss+100)*2+1
     for index, row in ref.iterrows():
         count = np.zeros(tss_range, dtype=np.int64)
         for frag in fs.fetch(row["seq_id"], row["start"], row["end"]):
@@ -126,9 +130,9 @@ def bedScan(args):
             dist_count.append(count)
     dist_count = np.array(dist_count)
     if dist_count.size > 0:
-        factors = np.mean(dist_count[:, list(range(0, args.range[1]))+list(range(tss_range-args.range[1], tss_range))], axis=1)
+        factors = np.mean(dist_count[:, list(range(0, 100))+list(range(tss_range-100, tss_range))], axis=1)
         factors[factors == 0] = np.mean(factors) if np.mean(factors) > 0 else 1
-        dist_count = pd.DataFrame({"V1": list(range(-args.range[0]+args.range[1], args.range[0]-args.range[1]+1)), "V2": list(np.mean(dist_count[:, args.range[1]:(tss_range-args.range[1])] / factors.reshape(len(factors), 1), axis=0))})
+        dist_count = pd.DataFrame({"V1": list(range(-args.widthtss, args.widthtss+1)), "V2": list(np.mean(dist_count[:, 100:(tss_range-100)] / factors.reshape(len(factors), 1), axis=0))})
     
     print("Saving the results...")
     if args.file_out:
@@ -157,11 +161,15 @@ def bedScan(args):
             width=6, height=6, dpi=200, filename=pathname+".tmpb.png", limitsize=False, verbose=False)
     if "c" in pic_list:
         break_length = 50
-        if args.range[0] - args.range[1] > 2000:
+        if args.widthtss > 6000:
+            break_length = 2000
+        if args.widthtss > 3000:
+            break_length = 1000
+        elif args.widthtss > 1000:
             break_length = 500
-        elif args.range[0] - args.range[1] > 400:
+        elif args.widthtss > 400:
             break_length = 200
-        break_range = max(1, (args.range[0]-args.range[1])//break_length)*break_length
+        break_range = max(1, args.widthtss//break_length)*break_length
         ggsave(plot=ggplot(dist_count, aes(x="V1", y="V2"))+geom_line(size=1, colour="#80B1D3")+
             labs(x="\nDistance from TSS (bp)", y="Mean TSS enrichment score")+
             scale_x_continuous(breaks=range(-break_range, break_range+1, break_length))+
@@ -184,8 +192,8 @@ def bedScan(args):
 
 def main():
     opts, args = getopt.getopt(sys.argv[1:], 
-        "hoi:r:g:m:q:l:f:c:p:n:b:", 
-        ["help", "output", "input=", "reference=", "group=", "mode=", "quality=", "length=", "filter=", "chr=", "pic=", "nl=","bg="])
+        "hoi:r:g:m:q:l:f:c:p:n:w:", 
+        ["help", "output", "input=", "reference=", "group=", "mode=", "quality=", "length=", "filter=", "chr=", "pic=", "nl=","widthtss="])
     arguments = ArgumentList()
     help_flag = False
     help_info = "Usage:\nATACFragQC [options] -i <input.bam> -r <reference.gtf>\nArguments:\n"\
@@ -201,7 +209,7 @@ def main():
         +"-n, --nl [0-X]\tThe length limit of chromosome names (default: 10, 0 is no limit)\n"\
         +"-p, --pic [a,b,c]\tThe list of images would be shown (default: all)\n"\
         +"-f, --filter [aaa,bbb]\tThe list of chromosomes which should be filtered (default: none)\n"\
-        +"-b, --bg RG,BG\tThe entire range and background of TSS enrichment (default: 1000,100)\n"
+        +"-w, --widthtss [100-10000]\tThe width of regions around the TSS (default: 900)\n"
     for opt, arg in opts:
         if opt in ("-h", "--help"):
             help_flag = True
@@ -229,11 +237,9 @@ def main():
             arguments.pic_list = arg
         elif opt in ("-n", "--nl"):
             arguments.cn_len = int(arg)
-        elif opt in ("-b", "--bg"):
-            if len(arg.split(",")) == 2:
-                arguments.range = list(map(lambda x: int(x), arg.split(",")))
-                if arguments.range[0] < arguments.range[1] or arguments.range[0] < 100 or arguments.range[1] < 3:
-                    arguments.range = [1000, 100]
+        elif opt in ("-w", "--widthtss"):
+            if int(arg) >= 100 and int(arg) <= 10000:
+                arguments.widthtss = int(arg)
     print("ATACFragQC - Version: "+__version__)
     if help_flag or arguments.file_bam == "" or arguments.file_ref == "":
         print(help_info)
